#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import time

import technoap.sitcp as sitcp
import technoap.device.apv8508 as apv8508
from technoap.device.apv8508 import constant


#device IP address
DEST_IPADDR = "192.168.10.128"

SAVE_FILE = "./hist.csv"

##########################################################
def main(args):
	
	try:
		rbcp = sitcp.rbcp.RBCP(True)		#sitcp(rbcp)
		rbcp.setdefaultdest(DEST_IPADDR)	#setting default destination

		config = apv8508.Config(rbcp)		#configuration
		status = apv8508.Status(rbcp)		#status read
		fasttx = apv8508.FastDataTX(rbcp)	#listdata read
		isConnect = fasttx.connectDevice(100, DEST_IPADDR)

		if isConnect == True:
			init_config(config)		#paramter configuration
			acquisition(config, status, fasttx, 5.0, SAVE_FILE)		#data acquisition
		else:
			print ("device(%s) connect error..." % (DEST_IPADDR))
	except Exception as ex:
		print (ex)
	

##########################################################
def acquisition(config, status, fasttx, acq_sec, savefile):
	
	dummy = config.stopAcquisition()		#Issuing a measurement stop command just in case, during measurement
	dummy = config.clearAcquisitionData()	#Clears the data remaining on the device during the last acquisition.

	dummy = config.setAcquisitionTime(acq_sec)							#acq time
	dummy = config.setAcquisitionMode(constant.REG_MOD_HIST)			#list mode
	dummy = config.setAcquisitionTimeMode(constant.REG_MMD_REALTIME)	#real time

	runAcq = True
	histData = [ None for n in range(fasttx.getChannels()) ]	#histogram data clear

	dummy = config.startAcquisition()		#start acquisition!!!

	while runAcq == True:
		status.readStatusAll()
		elapseTime = status.getElapsedSeconds()

		if elapseTime >= acq_sec:
			dummy = config.stopAcquisition()	#stop acquisition
			status.readStatusAll()	#Read the measured value after the measurement is completed
			runAcq = False
		
		#dump status
		print ("\n########## %.1f / %.1f ##########" % (elapseTime, acq_sec))
		print ("throughput total counts = %d, %d, %d, %d, %d, %d, %d, %d" % (
			status.mStatus["TCT",][0], status.mStatus["TCT",][1],
			status.mStatus["TCT",][2], status.mStatus["TCT",][3],
			status.mStatus["TCT",][4], status.mStatus["TCT",][5],
			status.mStatus["TCT",][6], status.mStatus["TCT",][7]))
		print ("throughput count rate   = %d, %d, %d, %d, %d, %d, %d, %d" % (
			status.mStatus["TCR",][0], status.mStatus["TCR",][1],
			status.mStatus["TCR",][2], status.mStatus["TCR",][3],
			status.mStatus["TCR",][4], status.mStatus["TCR",][5],
			status.mStatus["TCR",][6], status.mStatus["TCR",][7]))
		print ("dead time counts        = %d, %d, %d, %d, %d, %d, %d, %d" % (
			status.mStatus["DDT",][0], status.mStatus["DDT",][1],
			status.mStatus["DDT",][2], status.mStatus["DDT",][3],
			status.mStatus["DDT",][4], status.mStatus["DDT",][5],
			status.mStatus["DDT",][6], status.mStatus["DDT",][7]))
		

		########## save histogram ##########
		for i in range(fasttx.getChannels()):	#all CH
			temp0 = fasttx.readHistogram(i + 1)
			histData[i] = temp0

		time.sleep(1.0)	#wait(1000 msec)
	
	########## save histogram ##########
	with open(savefile, 'w') as csv:
		#header
		text  = '[header]'
		text += '\nreal time(sec),%f' % (acq_sec)
		text += '\nthroughput total count'
		for value in status.mStatus["TCT",]: text += ",%d" % (value)
		text += '\nthroughput count rate'
		for value in status.mStatus["TCR",]:  text += ",%d" % (value)
		csv.writelines(text + '\n\n')

		#data
		text  = '[data]\n'
		text += '#bin'
		for i in range(len(histData)): text += ",CH%d" % (i + 1)
		csv.writelines(text + '\n')

		for i in range(len(histData[0])):
			text = "%d" % (i)
			for j in range(len(histData)): text += ",%d" % (histData[j][i])
			csv.writelines(text + '\n')

	dummy = config.stopAcquisition()	#acq stop
	
	return


##########################################################
def init_config(config):

	config.clearStatus()

	########## common ##########
	#SPB: sampling bit(ADC)
	config.mStatus["SPB",] = constant.REG_SPB_12BIT		#12 bits
	#ARS: ADC read size byte
	config.mStatus["ARS",] = config.toDigitARS(constant.REG_ARS_256)
	#TGC: trigger sig
	config.mStatus["TGC",] = constant.REG_TGC_SIG1
	#TGE: trigger edge --> negative
	config.mStatus["TGE",] = constant.REG_TGE_POS
	#FRN: wave free run
	config.mStatus["FRN",] = constant.REG_FRN_ENABLE
	#FIT: calc FIFO IRQ trigger
	config.mStatus["FIT",] = constant.REG_FIT_DEFAULT
	#WTP: trigger point(=40), wave compress = 1/4, select sampling = 500MHz
	config.mStatus["WTP",] = config.toDigitWTP(40)

	########## CH ##########
	maxchn = constant.CHANNEL_MAX

	#Set the following parameters in the order of [CH1, CH2, CH3, CH4, CH5, CH6, CH7, CH8]
	#AFS: QDC full scale
	config.mStatus["AFS",] = [
		constant.REG_AFS_1_1, constant.REG_AFS_1_1,		#CH1: 1/1, CH2: 1/1
		constant.REG_AFS_1_1, constant.REG_AFS_1_1,		#CH3: 1/1, CH4: 1/1
		constant.REG_AFS_1_1, constant.REG_AFS_1_1,		#CH5: 1/1, CH6: 1/1
		constant.REG_AFS_1_1, constant.REG_AFS_1_1]		#CH7: 1/1, CH8: 1/1
	#GSL: analog gain --> all 3 times
	config.mStatus["GSL",] = [ constant.REG_GSL_X1 for n in range(maxchn) ]
	#RJT: raise judge threshold --> all 686 digit
	config.mStatus["RJT",] = [ 50 for n in range(maxchn) ]
	#POL: polarity
	config.mStatus["POL",] = [
		constant.REG_POL_POS, constant.REG_POL_POS,	#CH1: negative, CH2: negative
		constant.REG_POL_POS, constant.REG_POL_POS,	#CH3: negative, CH4: negative
		constant.REG_POL_POS, constant.REG_POL_POS,	#CH5: negative, CH6: negative
		constant.REG_POL_POS, constant.REG_POL_POS]	#CH7: negative, CH8: negative
	#CCF: CFD function --> all x0.21
	config.mStatus["CCF",] = [ constant.REG_CCF_X0_21 for n in range(maxchn) ]
	#CDL: CFD delay
	config.mStatus["CDL",] = [ constant.REG_CDL_10NS for n in range(maxchn) ]
	#CWK: CFD walk --> all 10(digit)
	config.mStatus["CWK",] = [ 10 for n in range(maxchn) ]
	#CTH: CFD threshold
	config.mStatus["CTH",] = [
		30, 30,	#CH1: 30 digit, CH2: 30 digit
		30, 30,	#CH3: 30 digit, CH4: 30 digit
		30, 30,	#CH5: 30 digit, CH6: 30 digit
		30, 30]	#CH7: 30 digit, CH8: 30 digit
	#CLD: QDC lower level discriminator --> all 10ch
	config.mStatus["CLD",] = [ 10 for n in range(maxchn) ]
	#CUD: QDC upper level discriminator --> all 8190ch
	config.mStatus["CUD",] = [ 8190 for n in range(maxchn) ]
	#FLK: baseline restorer
	config.mStatus["FLK",] = [
		constant.REG_FLK_4US, constant.REG_FLK_4US,	#CH1: 4usec, CH2: 4usec
		constant.REG_FLK_4US, constant.REG_FLK_4US,	#CH3: 4usec, CH4: 4usec
		constant.REG_FLK_4US, constant.REG_FLK_4US,	#CH5: 4usec, CH6: 4usec
		constant.REG_FLK_4US, constant.REG_FLK_4US]	#CH7: 4usec, CH8: 4usec
	#EPD: analog offset
	config.mStatus["EPD",] = [
		config.toDigitEPD(0.0), config.toDigitEPD(0.0),		#CH1: 0.0mv, CH2: 0.0mv
		config.toDigitEPD(0.0), config.toDigitEPD(0.0),		#CH3: 0.0mv, CH4: 0.0mv
		config.toDigitEPD(0.0), config.toDigitEPD(0.0),		#CH5: 0.0mv, CH6: 0.0mv
		config.toDigitEPD(0.0), config.toDigitEPD(0.0)]		#CH7: 0.0mv, CH8: 0.0mv
	#PTS: QDC pre-trigger --> all 0 nsec
	config.mStatus["PTS",] = [ constant.REG_PTS_P00NS for n in range(maxchn) ]
	#WSL: wave select
	v0, v1 = config.toDigitWSL(constant.REG_WCH_CH1, constant.REG_WTY_CFD)	#1st:
	v2, v3 = config.toDigitWSL(constant.REG_WCH_CH2, constant.REG_WTY_CFD)	#2nd:
	v4, v5 = config.toDigitWSL(constant.REG_WCH_CH3, constant.REG_WTY_CFD)	#3rd:
	v6, v7 = config.toDigitWSL(constant.REG_WCH_CH1, constant.REG_WTY_CFD)	#4th:
	config.mStatus["WSL",] = [ v0, v1, v2, v3, v4, v5, v6, v7 ]
	#LIG: QDC filter --> all 10 nsec
	config.mStatus["LIG",] = [ constant.REG_LIG_10NS for n in range(maxchn) ]
	#LIT: QDC select --> all sum
	config.mStatus["LIT",] = [ constant.REG_LIT_SUM for n in range(maxchn) ]
	#TTY: timing type --> all CFD
	config.mStatus["TTY",] = [ constant.REG_TTY_CFD for n in range(maxchn) ]
	#FST: FRC start count --> all 25
	config.mStatus["FST",] = [  25 for n in range(maxchn) ]
	#FSP: FRC stop count --> all 150
	config.mStatus["FSP",] = [ 150 for n in range(maxchn) ]
	#QIR: QDC integral range --> all 144 nsec
	config.mStatus["QIR",] = [ config.toDigitQIR(144) for n in range(maxchn) ]
	#WAS: input signale type
	config.mStatus["WAS",] = [ constant.REG_WAS_NORMALSIG for n in range(maxchn) ]

	#writing parameter
	result = config.writeStatusAll()
	time.sleep(1.0)		#waiting

	if result != True:
		raise Exception('config write error...')

	return result


##########################################################
if __name__=='__main__':
	main(sys.argv)

